<?php

class ChartData extends BaseObject {

 var $d_title;
 var $d_data;
 var $d_params;

 function ChartData($data, $params) {
 parent::BaseObject();
 $this->setData($data);
 $this->setParams($params);
 }

 function setTitle($title) {
 $this->d_title = $title;
 }

 function getTitle() {
 return $this->d_title;
 }

 function setParams( $params ) {
 if (!is_array($params)) return false;
 if (!is_array($this->d_params)) $this->d_params = array();
 $this->d_params = array_merge($this->d_params, $params);
 }

 function getParams() {
 return $this->d_params;
 }

 function setData( $data ) {
 $this->d_data = $data;
 }

 function getData() {
 return $this->d_data;
 }

}

class ChartRendererFactory extends BaseObject {

 function getCharts() {
 return array("gd2D"=>i18n("2D chart"), "java"=>i18n("java chart 3d"));
 }

 function getChartName($type) {
 $types = ChartRendererFactory::getCharts();
 if (!isset($types[$type])) return false;
 return ($types[$type])?$types[$type]:$type;
 }

 function &getInstanceOf($type, &$chartData) {
 $class = NULL;
 switch ($type) {
 case "gd2D": $class =& new GdChartRenderer($chartData, "2d"); break;
 case "java": $class =& new JavaChartRenderer($chartData ); break;
 default: $class = new NodeException("ChartRendererFactory: type ($type) does not exist.");
 }
 return $class;
 }

}

class ChartRenderer extends BaseObject {

 var $d_chartData;

 function ChartRenderer(&$chartData) {
 $this->setObjectRef("chartData", $chartData);
 }

 function &getChartData() {
 return $this->getObjectRef("chartData");
 }

 function getIcon() { return ""; }

 function getHtmlOutput() { return ""; }
}

class GdChartRenderer extends ChartRenderer {

 var $d_type;
 var $d_uniqueKey;

 function GdChartRenderer(&$chartData, $type) {
 parent::ChartRenderer($chartData);
 $this->d_type;
 }

 function getHtmlOutput() {
 global $STAG;
 
 if (!$this->d_uniqueKey) $this->d_uniqueKey = mysession_createUnique();
 mysession_updateUnique($this->d_uniqueKey, $this);
 
 $html = "<img src=\"$STAG&do=19&uniqueKey=".$this->d_uniqueKey."\" border=0 width=500 height=250>";
 return $html;
 }

 function getImageHandle() {
 node_include("chart2D");
 
 $oc =& $this->getChartData();
 $data = $oc->getData();
 $params = $oc->getParams();
 
 $d = array();
 $colors = array("00ee00","ee0000","eeee00","0000ee","00eeee","ee00ee"); 
 $i = 0;
 foreach($data[0] as $key=>$val) {
 $color = $colors[$i%count($colors)];
 $d[] = array(array($val["name"], ""), array((double)$val["value"], $color));
 $i++;
 }
 
 $chart = new chart2D($d);
 $chart->set("space", 0.2);
 $chart->set("x_size", 500);
 $chart->set("y_size", 250);
 $chart->set("x_title", $params["xTitle"]);
 $chart->set("y_title", $params["yTitle"]);
 $chart->set("x_left_margin", 80);
 $chart->set("col_back", "ffffff");
 $chart->set("border", 0);
 
 $chart->set("get_next_10_base", 0);
 $chart->set("get_next_10_div", 1);

 $chart->error_check();
 $img = $chart->render();
 
 return $img;
 }

 function printDiagram() {
 ImagePNG($this->getDiagramGDHandle());
 }

}

class JavaChartRenderer extends ChartRenderer {

 function getHtmlOutput() {
 $oc =& $this->getChartData();
 $data = $oc->getData();
 $params = $oc->getParams();
 $title = $oc->getTitle();
 
 if (!$title) $title = " ";
 
 $k = 1;
 $dval = array();
 $max_value = 0;
 if (is_array($data)) { 
 foreach ($data as $zKey=>$zVal) {
 if (!is_array($zVal)) continue;
 foreach ($zVal as $key=>$val) {
 $dnames .= $val["name"].";";
 $dparams .= "<param name=\"id$k\" value=\"$k|".$val["description"]."\">\n";
 $dval[$k] = $val["value"];
 $max_value = max($val["value"],$max_value);
 $k++;
 }
 }
 }
 
 $scale = 1;
 if ($max_value > 10000) {
 $scale = 1000;
 $title .= i18n(" in 1000");
 $max_value = sprintf("%1.0f", ($max_value * 1.1) / 1000);
 }
 while (list($k,$v) = each($dval)) {
 $dvalues .= sprintf("id%s:%1.0f;",$k, ($v/$scale));
 }
 
 
 $out = "<applet archive=\"tracking.files/javaStats_v0.99c.jar\" code=stats3D width=500 height=250>
<param name=\"list\" value=\"hide\">
$dparams
<param name=\"dia1\" value=\"$title|$dnames| |".$params["xTitle"]." |".$params["yTitle"]." | |0|$max_value|3|y|$dvalues\">
</applet>
		";
 
 return $out;
 }

}

?>
