<?php


function ldap_parse_sql($sql) {
 if (is_array($sql)) return $sql;

 $cmd = strtolower(substr($sql,0,6));
 
 $sp = "[ \r\n]";

 $arr = array();
 $arr["cmd"] = $cmd;
 
 if ($cmd == "delete") {
 
 if (!eregi("^delete$sp+from$sp+(.*[^,])($sp+where$sp+(.+))?$sp*$",$sql,$regs)) 
 halt(7,"mydb_ldap_exec: could not fetch sql ($sql)");


 }
 if ($cmd == "select") {
 
 if (!eregi("^select$sp+(.*[^,])$sp+from$sp+(.*[^,]?)$sp+where$sp+(.*)($sp+order$sp+by$sp+(.+))$sp*$",$sql,$regs)) 
 halt(7,"mydb_ldap_exec: could not fetch sql ($sql)");
 
 }
 
 return $arr;
}

function ldap_explode_attrs($attr) {
 $ret = array();

 if (is_array($attr)) {
 
 foreach ($attr as $key=>$val) {
 
 if (!is_array($val)) $val = array( $val );
 
 $ret[$key] = array_merge(array(), array_unique($val));
 }
 }

 return $ret;
}

function ldap_compile_sql($param) {

 if (!is_array($param)) return $param;
 
 $cmd = strtolower($param["cmd"]);
 
 $sql = strtoupper($param["cmd"])." ";
 
 if ($cmd == "insert" || $cmd == "update") {
 

 } else if ($cmd == "select") {
 
 $sql .= "*";
 $sql .= " FROM ".$param["basedn"]." WHERE ".$param["filter"];
 }
 
 return $sql;
}

function ldap_get_ldif($param) {

 $ldif = "dn: ".$param["dn"]."\n";
 foreach ($param["attr"] as $name=>$val) {
 if (is_array($val)) {
 foreach ($val as $v) 
 $ldif .= $name.": ".$v."\n";
 } else {
 $ldif .= $name.": ".$val."\n";
 }
 }
 return $ldif;
}

function mydb_ldap_connect($database, $server = "localhost",$login = "",$pw = "",$persistant = true, $halt = true) {
 global $FEATURE;
 
 $handle = false;
 if ($FEATURE["ldapv3"]) {
 ldap_set_option( $resource, LDAP_OPT_PROTOCOL_VERSION, 3 );
 }
 
 if (ereg("^([^:]+):([0-9]+)$",$server,$regs)) {
 $handle = ldap_connect($regs[1],$regs[2]);
 } else {
 
 $handle = ldap_connect($server);
 }
 if (!$handle && $halt) halt(7,"mydb_ldap_connect: could not connect to server ($server)");
 else if (!$handle) return false;

 if ($login) {
 $rs = @ldap_bind($handle,$login,$pw);
 } else {
 $rs = @ldap_bind($handle);
 }

 if (!$rs && $halt) halt(7, "mydb_ldap_connect: could not bind to server ($server) login($login) pw($pw) error(".ldap_error($handle).")");
 else if (!$rs) return false;

 return $handle;
}

function mydb_ldap_close($db) {
 return ldap_unbind($db);
}

function mydb_ldap_exec($sql, $db, $halt = true) {
 global $DB;
 
 if (!is_array($sql)) $sql = ldap_parse_sql($sql);
 
 $rs = false;

 if (is_array($db)) {
 $handle = $db;
 $db = $db["handle"];
 }
 
 $cmd = strtolower($sql["cmd"]);

 
 if ($cmd == "insert" || $cmd == "update") {
 
 $dn = $sql["dn"];
 $attr = ldap_explode_attrs($sql["attr"]);
 
 if (!$attr) 
 if ($halt) halt(7,"mydb_ldap_exec: no attrs for $cmd dn($dn) attr($attr)");
 else return false;

 if ($cmd == "insert") {
 $func = ($sql["object"])?"ldap_mod_add":"ldap_add";
 } else {
 $func = ($sql["object"])?"ldap_mod_replace":"ldap_modify";
 }

 
 if ( $sql['singlequery'] == true ) {
 $rs = $func($db, $dn, $attr );
 if (!$rs && $halt && ldap_errno($db)!= 20) halt(7, "error: mydb_ldap_exec error exec $cmd ($func). error (".ldap_error($db).")");
 } else {
 foreach ($attr as $key => $val) {
 foreach ($val as $vval) {
 $rs = $func($db, $dn, array($key => $vval) );
 if (!$rs && $halt && ldap_errno($db)!= 20) halt(7, "error: mydb_ldap_exec error exec $cmd ($func). error (".ldap_error($db).")");
 }
 }
 }
 }
 
 
 if ($cmd == "compare") {
 
 $dn = $sql["dn"];
 $attr = $sql["attr"];
 $val = $sql["val"];

 $rs = ldap_compare($db,$dn,$attr,$val);

 if ($halt && $rs == -1) halt(7,"error: mydb_ldap_exec: error exec compare. error (".ldap_error($db).")");
 }

 
 if ($cmd == "rename") {
 
 $dn = $sql["dn"];
 $newrdn = $sql["newrdn"];
 $newparent = $sql["newparent"];
 $deleteoldrdn = $sql["deleteoldrdn"] ? $sql["deleteoldrdn"] : false;

 $rs = ldap_rename($db,$dn,$newrdn,$newparent,true);
 if ($halt && !$rs) halt(7,"error: mydb_ldap_exec: error exec rename. error (".ldap_error($db).")");
 }

 
 if ($cmd == "delete") {
 
 $dn = $sql["dn"];
 $where = ldap_explode_attrs($sql["attr"]);

 
 if ($where) {
 foreach ($sql["attr"] as $key => $val) {
 foreach ($val as $vval) {
 $rs = ldap_mod_del($db,$dn, array($key => $vval) );
 }
 }
 } else {
 $rs = ldap_delete($db,$dn);
 }
 }

 
 if ($cmd == "select") {
 
 $basedn = $sql["basedn"];
 $filter = $sql["filter"];
 $orderby = $sql["orderby"];
 $type = strtolower($cmd["type"]);

 if ($type == "flat") $search = "ldap_list";
 else if ($type == "base") $search = "ldap_read";
 else $search = "ldap_search";
 
 $attr = $sql["attr"];

 if ($attr && !is_array($attr) && $attr != "*") {
 $attr = explode(",",$attr);
 } else if (!is_array($attr)) $attr = false;
 
 if ($attr) $res = $search($db, $basedn, $filter, $attr);
 else $res = $search($db, $basedn, $filter);
 
 if (!$res) halt(7,"mydb_ldap_exec: could not query sql (".ldap_compile_sql($sql)."), error(".ldap_error($db).")");
 
 if ($orderby) ldap_sort($db, $res, $orderby);
 
 $rs = array();
 $rs["db"] = $db;
 $rs["res"] = $res;
 $rs["pos"] = -1;
 }

 return $rs;
}

function mydb_ldap_fetch(&$rs,$way = "", $utf8decode = false) {
 if (!$rs["res"]) return false;
 if ($rs["pos"] < 0) {
 $e = ldap_first_entry($rs["db"],$rs["res"]);
 $rs["pos"] = 0;
 } else {
 $e = ldap_next_entry($rs["db"],$rs["last"]);
 $rs["pos"]++;
 }
 
 if (!$e) return false;
 
 $res = array();
 $res["dn"] = ldap_get_dn($rs["db"],$e);
 
 $attr = ldap_get_attributes($rs["db"],$e);
 for ($i = 0; $i < $attr["count"]; $i++) {
 $key = $attr[$i];
 $res[$key] = array();
 for ($j = 0; $j < $attr[$key]["count"];$j++) {
 if ( $utf8decode ) {
 $res[$key][] = utf8_decode($attr[$key][$j]);
 } else {
 $res[$key][] = $attr[$key][$j];
 }
 }
 }
 
 $rs["last"] = $e;
 return $res;
}

function mydb_ldap_query($sql, $cache = false, $limit = 0, $db = 0, $way = MYSQL_BOTH) {

 $rs = array();
 $rs["arr"] = array();
 $rs["iid"] = 0;
 $rs["num"] = 0;

 if (!is_array($sql)) $sql = ldap_parse_sql($sql);

 $res = mydb_ldap_exec($sql, $db);

 if (strtolower($sql["cmd"]) == "select") {
 while ($row = mydb_ldap_fetch($res, "", $sql['utf8decode'])) {
 $rs["num"]++;
 $rs["arr"][] = $row;
 }
 }

 return $rs;

}

function mydb_ldap_create_database($database, $db) {
 return false;
}

?>
