<?php

class ObjectEditor extends BaseObject {

 var $d_name;
 var $d_form;
 var $d_editObject;
 var $d_formObject;
 var $d_isDone;
 var $d_enableDelete;
 var $d_enableApply;
 var $d_actionPerformer;
 var $d_uniqueId;
 var $d_action;

 function ObjectEditor($keyname) {
 parent::BaseObject();
 $this->d_name = $keyname;
 $this->d_editObject = NULL;
 $this->d_formObject = NULL;
 $this->d_actionPerformer = array();
 $this->d_action = "";
 $this->d_enableApply = true;
 $this->d_enableDelete = false;
 $this->_restore();
 }

 function isNew() {
 return $this->d_form->isNew();
 }

 function setHeader($header, $icon = "") {
 $this->d_form->setHeader($header, $icon);
 }

 function isDone() {
 return $this->d_isDone;
 }

 function setEditObject(&$object) {
 unset($this->d_editObject);
 unset($this->d_formObject);
 
 if (is_a($object, "nodeobject") || is_a($object, "nodeobjectcollection")) {
 $this->d_editObject =& $object;
 $this->d_formObject =& $object->getFormObject();
 } else if (is_a($object, "nodeformobject") ){
 $this->d_editObject =& $object->getEditObject();
 $this->d_formObject =& $object;
 } else
 halt( 107, "ObjectEditor: edit object (".get_class($object).") is not a valid class" );
 
 $this->_updateObjectForm( $this->d_form );
 }

 function enableDelete($val) {
 $this->d_enableDelete = $val;
 $this->_updateButtons( $this->d_form );
 }

 function enableApply($val) {
 $this->d_enableApply = $val;
 $this->_updateButtons( $this->d_form );
 }
 
 function setOkAction(&$action) {
 $this->d_actionPerformer["ok"] =& $action;
 }

 function setCreateAction(&$action) {
 $this->d_actionPerformer["create"] =& $action;
 }

 function setUpdateAction(&$action) {
 $this->d_actionPerformer["update"] =& $action;
 }

 function setDeleteAction(&$action) {
 $this->d_actionPerformer["delete"] =& $action;
 }

 function setCancelAction(&$action) {
 $this->d_actionPerformer["cancel"] =& $action;
 }

 function _storeToSession() {

 $sPerf = array();
 foreach ($this->d_actionPerformer as $action=>$obj) {
 $sPerf[$action] = node_serialize($this->d_actionPerformer[$action]);
 }

 $rs = array("formObject"=>node_serialize($this->d_formObject),
 "performer"=>$sPerf,
 "enableDel"=>$this->d_enableDelete,
 "enableApply"=>$this->d_enableApply
 );
 mysession_updateUnique( $this->d_uniqueId, $rs );
 }

 function _restoreFromSession( $id ) {
 $this->d_uniqueId = $id;
 $rs = mysession_loadUnique( $id );

 $this->d_actionPerformer = array();
 foreach($rs["performer"] as $action=>$obj) {
 $this->d_actionPerformer[$action] = &node_unserialize($obj);
 }
 $this->d_formObject = &node_unserialize($rs["formObject"]);
 $this->d_editObject = &$this->d_formObject->getEditObject();
 $this->d_enableDelete = $rs["enableDel"];
 $this->d_enableApply = $rs["enableApply"];
 }

 function _restore() {
 unset( $this->d_form );
 $this->d_form =& new HtmlForm3R("oed".$this->d_name, $this);
 if ($this->d_form->isNew()) {
 $this->d_uniqueId = mysession_createUnique();
 } else {
 $comp = &$this->d_form->getComponent( "hidden" );
 $hidden = $comp->getValue();
 $this->_restoreFromSession( $hidden );
 }

 $this->_execAction( $this->d_form );
 }

 function _updateObjectForm( &$form ) {
 if (!is_object($this->d_editObject)) return false;

 $fields = $this->d_formObject->getValueFields();
 if ($fields === NULL) return false;

 $lbls = $this->d_formObject->getFieldLabels();

 $form->removeComponents();
 foreach ($fields as $field=>$type) {
 $comp =& $this->_getFormComponent($this->d_formObject, $field, $type, $lbls[$field]);
 if (!is_object($comp)) continue;
 $form->addComponent("f".$field, $comp); 
 }

 $form->addComponent( "hidden", new HiddenComponent( $this->d_uniqueId ) );

 $this->_updateButtons( $form );
 }

 function addComponent( $name, &$component )
 {
 $this->d_form->addComponent("c" . $name, $component);
 }

 function &getComponent( $name )
 {
 $this->d_form->getComponent( "c".$name );
 }

 function _updateButtons( &$form ) {
 if (!is_object($this->d_editObject)) return false;
 
 $form->removeButtons();
 
 $changeAllowed = $this->d_editObject->isNew() || $this->d_editObject->canUpdate();
 if ($changeAllowed) {
 $form->addButton("ok", new Button(i18n("OK")), new SimpleResolvableFormAction( "ok" ), TRUE);
 }
 $form->addButton("cancel", new Button(i18n("Cancel"), FALSE), new SimpleResolvableFormAction( "cancel" ));
 if (!$this->d_editObject->isNew() && $this->d_enableDelete) {
 $form->addButton("delete", new Button(i18n("Delete"), FALSE, $this->d_editObject->canDelete()), new SimpleResolvableFormAction( "delete" ));
 }
 if ($changeAllowed && $this->d_enableApply) {
 $form->addButton("apply", new Button(i18n("Apply")), new SimpleResolvableFormAction( "apply" ));
 }
 }

 function &_getFormComponent(&$obj, $field, $type, $label) {
 if (!$type) {
 $ret =& $obj->getFormComponent($field);
 return $ret;
 }
 
 $optional = $obj->isOptionalField($field);
 $editable = $obj->isEditableField($field);
 $value = $obj->getFieldValue($field);
 
 switch ($type) {
 case "realpath":
 node_class("textcomponent");
 $comp = new TextComponent($label, $editable, $value, REG_REAL_PATH, 20, $optional);
 break;
 case "text":
 node_class("textcomponent");
 $comp = new TextComponent($label, $editable, $value, REG_TEXT, 20, $optional);
 break;
 case "textarea":
 node_class("textareacomponent");
 $comp = new TextAreaComponent($label, $editable, $value, REG_TEXT, $optional);
 break;
 case "url":
 $comp = new TextComponent($label, $editable, $value, REG_URL, 20, $optional);
 break;
 }
 
 return $comp;
 }

 function _storeFormComponent(&$obj, $field, $type, &$comp) {
 if (!$type) {
 return $obj->storeFormComponent($field, $comp);
 }

 switch ($type) {
 case "text":
 case "realpath":
 case "textarea":
 case "url":
 $obj->setFieldValue($field, $comp->getText());
 break;
 }
 }

 function resolveAction(&$form, &$action) {
 $this->d_action = $action->getName();

 }

 function _execAction( &$form ) {
 switch ($this->d_action) {
 case "ok":
 case "apply":
 $action = $this->d_editObject->isNew()?"create":"update";

 $fields = $this->d_formObject->getValueFields();
 if ($fields === NULL) return false;

 foreach ($fields as $field=>$type) {
 if (!$form->componentExists("f".$field)) continue;
 $comp =& $form->getComponent("f".$field);
 $err = $this->_storeFormComponent($this->d_formObject, $field, $type, $comp);
 }
 
 $saveSuccess = $this->d_formObject->save();
 if ($this->_checkErrors($form) && $saveSuccess) {
 $this->_performAction($form, $action);
 $this->_performAction($form, $this->d_action);
 }
 break;
 case "delete":
 $this->d_editObject->delete();
 if ($this->_checkErrors($form)) {
 $this->_performAction($form, "delete");
 }
 break;
 case "cancel":
 $this->_performAction($form, "cancel");
 break;
 }
 
 }

 function _checkErrors(&$form) {

 $error = true;
 if ($err = $this->d_editObject->getErrors()) {
 foreach ( $err as $field=>$exception ) {
 unset( $class );
 $class =& $form->resolveComponent("f".$field);
 if (!is_a($class,"formcomponent")) {
 unset($class); $class =& $form->getComponent("hidden");
 }
 if ( is_array( $exception ) && count( $exception ) > 0 )
 {
 $class->addError($exception[0]);
 }
 $error = false;
 }
 }
 
 return $error;
 }

 function _performAction(&$form, $type) {
 if (is_object($this->d_actionPerformer[$type])) {
 $this->d_actionPerformer[$type]->perform($form, NULL);
 } else {
 
 }
 }

 function getHtmlOutput() {
 
 $ret = $this->d_form->getHtmlOutput();
 
 $this->_storeToSession();

 return $ret;
 }

}

?>
